/*
 * Copyright (c) 2017, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

/*
 * @test
 * @bug 8130072
 * @summary Check that Shared Dictionary is printed out with jcmd
 * Feature support: compressed oops/kptrs, 64-bit os, not on windows
 * @requires vm.cds
 * @requires (sun.arch.data.model != "32") & (os.family != "windows")
 * @requires (vm.opt.UseCompressedOops == null) | (vm.opt.UseCompressedOops == true)
 * @library /test/lib
 * @modules java.base/jdk.internal.misc
 * @run main DumpSharedDictionary
 */

import jdk.test.lib.cds.CDSTestUtils;
import jdk.test.lib.process.ProcessTools;
import jdk.test.lib.process.OutputAnalyzer;
import jdk.test.lib.JDKToolFinder;

public class DumpSharedDictionary {

    public static void main(String[] args) throws Exception {
        if (args.length == 0) {
            // Start this process
            ProcessBuilder pb = ProcessTools.createJavaProcessBuilder(
                "-XX:+UnlockDiagnosticVMOptions",
                "-XX:SharedArchiveFile=./DumpSharedDictionary.jsa",
                "-XX:+UseCompressedOops",
                "-Xshare:dump");

            OutputAnalyzer out = CDSTestUtils.executeAndLog(pb, "dump");
            out.shouldHaveExitValue(0);

            String testjdkPath = System.getProperty("test.jdk");

            pb = ProcessTools.createJavaProcessBuilder(
                    "-XX:+UnlockDiagnosticVMOptions",
                    "-XX:SharedArchiveFile=./DumpSharedDictionary.jsa",
                    "-XX:+UseCompressedOops",
                    "-Dtest.jdk=" + testjdkPath,
                    "-Xshare:on", "DumpSharedDictionary", "test");

            out = CDSTestUtils.executeAndLog(pb, "exec");
            if (!CDSTestUtils.isUnableToMap(out)) {
                out.shouldHaveExitValue(0);
            }
        } else {
            // Grab my own PID
            String pid = Long.toString(ProcessTools.getProcessId());

            ProcessBuilder pb = new ProcessBuilder();
            pb.command(new String[] {JDKToolFinder.getJDKTool("jcmd"), pid, "VM.systemdictionary"});
            OutputAnalyzer output = CDSTestUtils.executeAndLog(pb, "jcmd-systemdictionary");
            try {
                output.shouldContain("Shared Dictionary statistics:");
                output.shouldContain("Number of buckets");
                output.shouldContain("Number of entries");
                output.shouldContain("Maximum bucket size");
            } catch (RuntimeException e) {
                output.shouldContain("Unknown diagnostic command");
            }

            pb.command(new String[] {JDKToolFinder.getJDKTool("jcmd"), pid, "VM.systemdictionary", "-verbose"});
            output = CDSTestUtils.executeAndLog(pb, "jcmd-systemdictionary-verbose");
            try {
                output.shouldContain("Shared Dictionary");
                output.shouldContain("Dictionary for class loader 0x");
                output.shouldContain("^java.lang.String");
            } catch (RuntimeException e) {
                output.shouldContain("Unknown diagnostic command");
            }
        }
    }
}
